/*
Reads a rotary encoder and shows the position on a OLED-128-G1
display from 4D Systems as a number and a bargraph, and shows a clock.
*/

#include <stdint.h>
#include "hardware.h"
#include "display.h"

// current rotary encoder position
static volatile uint8_t g_position = 0;

// counts from 0 to 999
static volatile uint16_t g_millisecondCounter = 0;

// current time for the clock
static volatile uint8_t g_secondCounter = 0;
static volatile uint8_t g_minuteCounter = 0;
static volatile uint8_t g_hourCounter = 0;

// table for all state transitions of the rotary encoder inputs,
// mapped to the counting direction
// A oldA B oldB counter
// 0 0    0 0    0
// 0 0    0 1    1
// 0 0    1 0    -1
// 0 0    1 1    0
// 0 1    0 0    -1
// 0 1    0 1    0
// 0 1    1 0    0
// 0 1    1 1    1
// 1 0    0 0    1
// 1 0    0 1    0
// 1 0    1 0    0
// 1 0    1 1    -1
// 1 1    0 0    0
// 1 1    0 1    -1
// 1 1    1 0    1
// 1 1    1 1    0
#if 0
// table of a perfect rotary encoder, without bouncing
static const int8_t g_delta2[16] = {
  0, 1, -1, 0, -1, 0, 0, 1, 1, 0, 0, -1, 0, -1, 1, 0
};
#else
// for ALPS EC11 und EC12 Encoder:
// Ignore changes on B and use changes on A, only, if B=0
// this results in one step per raster and no bouncing between two values because of B
// see http://www.mikrocontroller.net/articles/Drehgeber for details
static const int8_t g_delta[16] = {
  0, 0, 0, 0, -1, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0
};
#endif

// returns the direction the rotary encoder was rotated: -1, 1 oder 0
static int8_t getDelta(uint8_t a, uint8_t oldA, uint8_t b, uint8_t oldB)
{
  return g_delta[oldB | (b << 1) | (oldA << 2) | (a << 3)];
}

// is called with 1 kHz
void onInterrupt(void)
{
	// evaluate rotary encoder
	static uint8_t oldA = 0;
	static uint8_t oldB = 0;
	uint8_t a = getA();
	uint8_t b = getB();
	int8_t delta = getDelta(a, oldA, b, oldB);
	if (delta < 0) g_position--;
	if (delta > 0) g_position++;
	oldA = a;
	oldB = b;
	
	// increment millisecond counter
	g_millisecondCounter++;
	if (g_millisecondCounter == 1000) {
		g_millisecondCounter = 0;
		g_secondCounter++;
		if (g_secondCounter == 60) {
			g_secondCounter = 0;
			g_minuteCounter++;
			if (g_minuteCounter == 60) {
				g_minuteCounter = 0;
				g_hourCounter++;
				if (g_hourCounter == 23) {
					g_hourCounter = 0;
				}
			}
		}
	}
}

// main program
int main(void)
{
	// init hardware and display
	initHardware();
	initDisplay();
	
	// print static text
	drawString(0, 0, COLOR_RED, 1, "R");
	drawString(10, 0, COLOR_GREEN, 1, "G");
	drawString(20, 0, COLOR_BLUE, 1, "B");
	drawString(0, 20, COLOR_LIGHT_SEA_GREEN, 2, "Position");
	
	// endless loop: show rotary encoder position and clock
	while (1) {
		// disable interrupt and read current position and millisecond counter
		disableInterrupts();
		uint16_t milliseconds = g_millisecondCounter;
		uint8_t newPosition = g_position;
		enableInterrupts();
		
		// show rotary encoder value with 1 Hz blinking
		uint32_t color;
		if (milliseconds > 500) {
			color = milliseconds >> 3;
		} else {
			color = (1000 - milliseconds) >> 3;
		}
		color &= 0xff;
		color <<= 8;
		drawInt(0, 50, color, 5, newPosition);
		
		// show value as bargraph
		uint8_t pos = newPosition;
		if (pos > 128) pos = 128;
		drawRectangle(0, 110, pos, 128, COLOR_YELLOW);
		drawRectangle(pos + 1, 110, 128, 128, COLOR_BLACK);
		
		// show current time
		drawTime(60, 0, COLOR_WHITE, g_hourCounter, g_minuteCounter, g_secondCounter);
	}
}
